extern crate libc;

use libc::{ c_char, c_int, c_void, size_t };
use std::ffi::{ CStr, CString };
use std::ptr;

#[link(name = "p4pillondebuglib")]
extern "C" {
    fn hello() -> *const c_char;
    fn helloName(name: *const c_char) -> *const c_char;
    fn helloPtr(result: *mut c_char);
    fn helloPtrPtr(result: *mut *mut c_char, size: *mut size_t);
    fn helloVoidPtrPtr(result: *mut *mut c_void, size: *mut size_t);
    fn add(a: c_int, b: c_int) -> c_int;
    fn addPtr(a: c_int, b: c_int, result: *mut c_int);
    fn fillHexValues(result: *mut *mut c_void, size: *mut size_t);
}

pub fn demo() {
    println!("------- Demo for the P4PillonDebugLib ---");
    
    unsafe {
        let c_str = hello();
        let r_str: &str = CStr::from_ptr(c_str).to_str().unwrap();
        println!("{} from C", r_str);
    }

    let name = CString::new("John").expect("CString::new failed");
    unsafe {
        let c_str = helloName(name.as_ptr());
        let r_str = CStr::from_ptr(c_str).to_str().expect("Conversion failed");
        println!("{} from C with name", r_str);

        libc::free(c_str as *mut c_void);
    }

    let mut buffer: [c_char; 50] = [0; 50];
    unsafe {
        helloPtr(buffer.as_mut_ptr());
        let c_str = CStr::from_ptr(buffer.as_ptr());
        let r_str = c_str.to_str().unwrap();
        println!("{} from C Ptr", r_str);
    }

    let mut buffer: *mut c_char = ptr::null_mut();
    let mut size: size_t = 0;
    unsafe {
        helloPtrPtr(&mut buffer, &mut size);
        
        if !buffer.is_null() {
            let c_str = CStr::from_ptr(buffer);
            let r_str = c_str.to_str().unwrap();
            println!("{} from C Ptr Ptr (size: {})", r_str, size);

            libc::free(buffer as *mut c_void);
        }
    }

    let mut buffer: *mut c_void = ptr::null_mut();
    let mut size: size_t = 0;
    unsafe {
        helloVoidPtrPtr(&mut buffer, &mut size);
        
        if !buffer.is_null() {
            let c_str = CStr::from_ptr(buffer as *const c_char);
            let r_str = c_str.to_str().unwrap();
            println!("{} from C Void Ptr Ptr (size: {})", r_str, size);

            libc::free(buffer);
        }
    }

    unsafe {
        let result = add(1, 2);
        println!("Result of 1 + 2 = {}", result);
    }

    unsafe {
        let mut result = 0;
        addPtr(1, 2, &mut result);
        println!("Result Ptr of 1 + 2 = {}", result);
    }

    let mut buffer: *mut c_void = ptr::null_mut();
    let mut size: size_t = 0;
    unsafe {
        fillHexValues(&mut buffer, &mut size);
        
        if !buffer.is_null() {
            let hex_values = std::slice::from_raw_parts(buffer as *const u8, size);
            for &byte in hex_values {
                print!("{:02X} ", byte);
            }
            println!();

            libc::free(buffer);
        }
    }

    println!("-----------------------------------------");
}
